#include "..\avisynth.h"

#define max(a,b) (a) > (b) ? (a) : (b)

class StackVert : public IClip {
  PClip clip1, clip2;
  VideoInfo vi;

public:
  StackVert(PClip _clip1, PClip _clip2, IScriptEnvironment* env);
  ~StackVert();
  PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
  void __stdcall GetAudio(void *buf, int start, int count, IScriptEnvironment* env);
  const VideoInfo& __stdcall GetVideoInfo();
  bool __stdcall GetParity(int n);

};
/************************************************************************/
//
StackVert::StackVert(PClip _clip1, PClip _clip2, IScriptEnvironment* env)
: clip1(_clip1), clip2(_clip2)
{
  if (_clip1->GetVideoInfo().IsYUY2()) {
	clip1 = _clip1; clip2 = _clip2;
  } else {
	clip1 = _clip2; clip2 = _clip1;
  }

  const VideoInfo& vi1 = clip1->GetVideoInfo();
  const VideoInfo& vi2 = clip2->GetVideoInfo();

  if (vi1.width != vi2.width)
	env->ThrowError("StackVertical: image widths don't match");
  if (vi1.pixel_type != vi2.pixel_type)
	env->ThrowError("StackVertical: image formats don't match");

  vi = vi1;
  vi.height += vi2.height;
  vi.num_frames = max(vi1.num_frames, vi2.num_frames);
  vi.num_audio_samples = max(vi1.num_audio_samples, vi2.num_audio_samples);

}
/************************************************************************/
//
StackVert::~StackVert()
{
}
/************************************************************************/
//
/*
AVSValue __cdecl StackVert::Create_StackVert(AVSValue args, void*, IScriptEnvironment* env) {
	PClip result = args[0].AsClip();
	for (int i=0; i<args[1].ArraySize(); ++i)
	  result = new StackVert(result, args[1][i].AsClip(), env);
	return result;
  }
*/
AVSValue __cdecl Create_StackVert(AVSValue args, void* user_data, IScriptEnvironment* env) {
  return new StackVert(args[0].AsClip(), args[1].AsClip(), env);
}

/************************************************************************/
//
PVideoFrame __stdcall StackVert::GetFrame(int n, IScriptEnvironment* env)
{
  PVideoFrame src1 = clip1->GetFrame(n, env);
  PVideoFrame src2 = clip2->GetFrame(n, env);
  PVideoFrame dst = env->NewVideoFrame(vi);
  const unsigned char* src1p = src1->GetReadPtr();
  const unsigned char* src2p = src2->GetReadPtr();
  unsigned char* dstp = dst->GetWritePtr();
  const int src1_pitch = src1->GetPitch();
  const int src2_pitch = src2->GetPitch();
  const int dst_pitch = dst->GetPitch();
  const int src1_height = src1->GetHeight();
  const int src2_height = src2->GetHeight();
  const int row_size = dst->GetRowSize();
  unsigned char* dstp2 = dstp + dst_pitch * src1->GetHeight();

  env->BitBlt(dstp, dst_pitch, src1p, src1_pitch, row_size, src1_height);
  env->BitBlt(dstp2, dst_pitch, src2p, src2_pitch, row_size, src2_height);

  return dst;

}
/************************************************************************/
//
void __stdcall StackVert::GetAudio(void *buf, int start, int count, IScriptEnvironment* env)
{
  clip1->GetAudio(buf, start, count, env);
}
/************************************************************************/
//
/*
StackVert::GetParity(int n)
{
}
*/
// return field parity if field_based, else parity of first field in frame
bool __stdcall StackVert::GetParity(int n)
{
  return clip1->GetParity(n);
}

/************************************************************************/
//
/*
StackVert::GetVersion()
{
}
*/
/************************************************************************/
//
const VideoInfo& __stdcall StackVert::GetVideoInfo()
{ return vi; }
/************************************************************************/
//
/************************************************************************/
//
/************************************************************************/
//
/************************************************************************/
//
